@@ -38,7 +38,59 @@
 # http://www.1000genomes.org/wiki/analysis/variant-call-format
 #
 
-seqVCF_Header <- function(vcf.fn, getnum=FALSE, verbose=TRUE)
+.count_vcf_samtools <- function(fn, parallel=FALSE)
+{
+    # check
+    stopifnot(is.character(fn), length(fn)==1L, !is.na(fn))
+    if (!requireNamespace("Rsamtools", quietly=TRUE))
+        stop("Rsamtools should be installed when 'parallel' is not FALSE.")
+    # check the indexing file
+    idxfn <- paste0(fn, ".csi")
+    if (!file.exists(idxfn))
+    {
+        idxfn <- paste0(fn, ".tbi")
+        if (!file.exists(idxfn))
+            stop("The indexing file should exist (either .csi or .tbi) when 'parallel' is used.")
+    }
+    # process
+    pnum <- .NumParallel(parallel)
+    if (pnum<=1L || isTRUE(file.size(fn) <= 67108864L))
+    {
+        # if file size <= 64MB
+        f <- Rsamtools::TabixFile(fn, idxfn)
+        open(f)
+        on.exit(close(f))
+        unlist(Rsamtools::countTabix(f), use.names=FALSE)
+    } else {
+        f <- Rsamtools::TabixFile(fn, idxfn)
+        open(f)
+        s <- Rsamtools::seqnamesTabix(f)
+        close(f)
+        # generate Granges object
+        if (length(s) < pnum)
+        {
+            each <- 5000000L
+            p <- (seq_len(100L)-1L) * each + 1L
+            r <- IRanges::IRanges(p, width=each)
+            gr <- GenomicRanges::GRanges(rep(s, each=length(r)),
+                rep(r, length(s)))
+        } else {
+            gr <- GenomicRanges::GRanges(s, IRanges::IRanges(1L, 2^29))
+        }
+        # parallel
+        lst <- seqParApply(parallel, 1:length(gr),
+            FUN=function(i, vcffn, idxfn, gr)
+            {
+                f <- Rsamtools::TabixFile(vcffn, idxfn)
+                open(f); on.exit(close(f))
+                unlist(Rsamtools::countTabix(f, param=gr[i,]), use.names=FALSE)
+            }, vcffn=fn, idxfn=idxfn, gr=gr)
+        do.call(sum, lst)
+    }
+}
+
+seqVCF_Header <- function(vcf.fn, getnum=FALSE, use_Rsamtools=NA,
+    parallel=FALSE, verbose=TRUE)
 {
     # check
     if (!inherits(vcf.fn, "connection"))
@@ -49,7 +101,10 @@ seqVCF_Header <- function(vcf.fn, getnum=FALSE, verbose=TRUE)
         ilist <- 1L
     }
     stopifnot(is.logical(getnum), length(getnum)==1L)
+    stopifnot(is.logical(use_Rsamtools), length(use_Rsamtools)==1L)
     stopifnot(is.logical(verbose), length(verbose)==1L)
+    if (getnum)
+        njobs <- .NumParallel(parallel)
 
     #########################################################
     # open the vcf file
@@ -115,8 +170,21 @@ seqVCF_Header <- function(vcf.fn, getnum=FALSE, verbose=TRUE)
                     }
                     if (isTRUE(getnum))
                     {
-                        nVariant <- nVariant + length(s) +
-                            .Call(SEQ_VCF_NumLines, infile, FALSE, verbose)
+                        call_count_vcf <- FALSE
+                        if (njobs > 1L)
+                        {
+                            if (is.na(use_Rsamtools))
+                                use_Rsamtools <- requireNamespace("Rsamtools", quietly=TRUE)
+                            if (use_Rsamtools) call_count_vcf <- TRUE
+                        }
+                        if (call_count_vcf)
+                        {
+                            nVariant <- nVariant +
+                                .count_vcf_samtools(vcf.fn[i], parallel)
+                        } else {
+                            nVariant <- nVariant + length(s) +
+                                .Call(SEQ_VCF_NumLines, infile, FALSE, verbose)
+                        }
                     }
                 }
                 break
@@ -486,7 +554,8 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
     storage.option="LZMA_RA", info.import=NULL, fmt.import=NULL,
     genotype.var.name="GT", ignore.chr.prefix="chr",
     scenario=c("general", "imputation"), reference=NULL, start=1L, count=-1L,
-    optimize=TRUE, raise.error=TRUE, digest=TRUE, parallel=FALSE, verbose=TRUE)
+    variant_count=NA_integer_, optimize=TRUE, raise.error=TRUE, digest=TRUE,
+    use_Rsamtools=NA, parallel=FALSE, verbose=TRUE)
 {
     # check
     if (!inherits(vcf.fn, "connection"))
@@ -525,29 +594,27 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
     stopifnot(is.logical(optimize), length(optimize)==1L)
     stopifnot(is.logical(raise.error), length(raise.error)==1L)
     stopifnot(is.logical(digest) | is.character(digest), length(digest)==1L)
+    stopifnot(is.logical(use_Rsamtools), length(use_Rsamtools)==1L)
     stopifnot(is.logical(verbose), length(verbose)==1L)
+    show_timeheader <- !isTRUE(attr(verbose, "header_no_time"))
 
-    pnum <- .NumParallel(parallel)
     parallel <- .McoreParallel(parallel)
+    pnum <- .NumParallel(parallel)
+
     if (inherits(vcf.fn, "connection"))
     {
         if (pnum > 1L)
             stop("No parallel support when the input is a connection object.")
-    }
-
-    if (is.character(vcf.fn))
-        variant_count <- attr(vcf.fn, "variant_count")
-    else
-        variant_count <- NULL
-    if (!is.null(variant_count))
+        if (length(variant_count)!=1L || !is.na(variant_count))
+            warning("'variant_count' is not used in seqVCF2GDS() when 'vcf.fn' is a connection object.")
+    } else if (!identical(variant_count, NA_integer_))
     {
         if (!is.numeric(variant_count))
-            stop("the attribute 'variant_count' of 'vcf.fn' should be a numeric vector.")
+            stop("'variant_count' should be a numeric vector.")
         if (length(variant_count) != length(vcf.fn))
-            stop("the attribute 'variant_count' of 'vcf.fn' should be as the same length as 'vcf.fn'.")
+            stop("'variant_count' and 'vcf.fn' should have the same length.")
     }
-
-    if (verbose) cat(date(), "\n", sep="")
+    if (verbose && show_timeheader) .cat("##< ", .tm())
 
     genotype.storage <- "bit2"
 
@@ -561,8 +628,8 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
             if (is.null(samp.id))
             {
                 samp.id <- seqVCF_SampID(vcf.fn[i])
-                if (length(samp.id) <= 0L)
-                    message("No sample in '", vcf.fn[i], "'")
+                # if (length(samp.id) <= 0L)
+                #     message("No sample in '", vcf.fn[i], "'")
             } else {
                 tmp <- seqVCF_SampID(vcf.fn[i])
                 if (length(samp.id) != length(tmp))
@@ -635,7 +702,6 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
         if (!is.character(storage.tmp))
             storage.tmp <- "customized"
         .cat("    compression method: ", storage.tmp)
-        .cat("    # of samples: ", length(header$sample.id))
         if (identical(scenario, "imputation"))
         {
             cat("    scenario: imputation\n")
@@ -717,17 +783,36 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
     {
         if (verbose)
         {
-            cat("    calculating the total number of variants ...\n")
+            .cat("    # of cores/jobs: ", pnum)
+            a <- variant_count < 0L
+            if (length(a) < length(vcf.fn)) a[length(vcf.fn)] <- NA
+            if (anyNA(a) || any(a, na.rm=TRUE))
+            {
+                cat("    calculating the total number of variants")
+                if (!isFALSE(use_Rsamtools))
+                {
+                    if (requireNamespace("Rsamtools", quietly=TRUE))
+                        cat(" using Rsamtools")
+                }
+                cat(" ...\n")
+            }
             flush.console()
         }
 
         # get the number of variants in each VCF file
-        num_array <- unlist(seqParApply(parallel, vcf.fn, function(fn)
+        for (i in seq_along(vcf.fn))
         {
-            library(SeqArray, quietly=TRUE, verbose=FALSE)
-            seqVCF_Header(fn, getnum=TRUE)$num.variant
-        }))
-        num_var <- sum(num_array)
+            v <- variant_count[i]
+            if (is.na(v) || (v < 0L))
+            {
+                fn <- vcf.fn[i]
+                variant_count[i] <- seqVCF_Header(fn, getnum=TRUE,
+                    use_Rsamtools=use_Rsamtools, parallel=parallel,
+                    verbose=FALSE)$num.variant
+            }
+        }
+        num_var <- sum(variant_count)
+        if (anyNA(num_var)) stop("Getting invalid # of variants.")
 
         if (start < 1L)
             stop("'start' should be a positive integer if conversion in parallel.")
@@ -738,7 +823,7 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
         if (start+count > num_var+1L)
             stop("Invalid 'count'.")
         if (verbose)
-            cat("    # of variants: ", .pretty(count), "\n", sep="")
+            .cat("    # of variants: ", .pretty(count))
 
         if (count >= pnum)
         {
@@ -748,45 +833,68 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
             psplit <- .file_split(count, pnum, start)
             if (verbose)
             {
-                cat(sprintf("    >>> writing to %d files: <<<\n", pnum))
+                .cat("    >>> writing to ", pnum, " files: <<<")
                 cat(sprintf("        %s\t[%s .. %s]\n", basename(ptmpfn),
                     .pretty(psplit[[1L]]),
                     .pretty(psplit[[1L]] + psplit[[2L]] - 1L)), sep="")
                 flush.console()
             }
+            # unlimit the last one
+            psplit[[2L]][length(psplit[[2L]])] <- -1L
+
+            # show information
+            update_info <- function(i)
+            {
+                .cat("        |> ", i, " [", .tm(), " done]")
+                flush.console()
+                NULL
+            }
+            if (!isTRUE(verbose)) update_info <- "none"
+
+            # reset memory before calling parallel
+            gc(FALSE, reset=TRUE, full=TRUE)
 
             # conversion in parallel
             seqParallel(parallel, NULL, FUN = function(
                 vcf.fn, header, storage.option, info.import, fmt.import,
                 genotype.var.name, ignore.chr.prefix, scenario, optim,
-                raise.err, ptmpfn, psplit, num_array)
+                raise.err, ptmpfn, psplit, variant_count)
             {
-                # load package
-                library(SeqArray, quietly=TRUE, verbose=FALSE)
-
-                attr(vcf.fn, "variant_count") <- num_array
                 i <- process_index  # the process id, starting from one
-
-                seqVCF2GDS(vcf.fn, ptmpfn[i], header=oldheader,
-                    storage.option=storage.option, info.import=info.import,
-                    fmt.import=fmt.import, genotype.var.name=genotype.var.name,
-                    ignore.chr.prefix=ignore.chr.prefix,
-                    start = psplit[[1L]][i], count = psplit[[2L]][i],
-                    optimize=optim, scenario=scenario, raise.error=raise.err,
-                    digest=FALSE, parallel=FALSE, verbose=FALSE)
-
-                invisible()
-
-            }, split="none",
+                tryCatch(
+                {
+                    SeqArray::seqVCF2GDS(vcf.fn, ptmpfn[i], header=oldheader,
+                        storage.option=storage.option, info.import=info.import,
+                        fmt.import=fmt.import,
+                        genotype.var.name=genotype.var.name,
+                        ignore.chr.prefix=ignore.chr.prefix,
+                        start = psplit[[1L]][i], count = psplit[[2L]][i],
+                        variant_count=variant_count,
+                        optimize=optim, scenario=scenario,
+                        raise.error=raise.err,
+                        digest=FALSE, parallel=FALSE, verbose=FALSE)
+                    i  # return the process index
+                }, error = function(e) {
+                    # capture full traceback
+                    trace <- capture.output({
+                        cat("Error: ", e$message, "\n", sep="")
+                        traceback()
+                    })
+                    con <- file(paste0(ptmpfn[i], ".progress.txt"), open="at")
+                    writeLines(trace, con)
+                    close(con)
+                    stop(e$message)
+                })
+            }, split = "none", .combine = update_info,
                 vcf.fn=vcf.fn, header=header, storage.option=storage.option,
                 info.import=info.import, fmt.import=fmt.import,
                 genotype.var.name=genotype.var.name,
                 ignore.chr.prefix=ignore.chr.prefix, scenario=scenario,
                 optim=optimize, raise.err=raise.error,
-                ptmpfn=ptmpfn, psplit=psplit, num_array=num_array)
+                ptmpfn=ptmpfn, psplit=psplit, variant_count=variant_count)
 
             if (verbose)
-                cat("    >>> Done (", date(), ") <<<\n", sep="")
+                .cat("    >>> Done (", .tm(), ") <<<")
 
         } else {
             pnum <- 1L
@@ -796,7 +904,7 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
 
 
     #######################################################################
-    # create a GDS file
+    # create a new GDS file
 
     gfile <- createfn.gds(out.fn)
     on.exit({ if (!is.null(gfile)) closefn.gds(gfile) }, add=TRUE)
@@ -849,13 +957,13 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
     # add variant.id
     .AddVar(storage.option, gfile, "variant.id", storage="int32")
 
+    # add chromosome
+    .AddVar(storage.option, gfile, "chromosome", storage="string")
+
     # add position
     # TODO: need to check whether position can be stored in 'int32'
     .AddVar(storage.option, gfile, "position", storage="int32")
 
-    # add chromosome
-    .AddVar(storage.option, gfile, "chromosome", storage="string")
-
     # add allele
     .AddVar(storage.option, gfile, "allele", storage="string")
 
@@ -1142,15 +1250,15 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
         linecnt <- double(1L)
 
         # progress file
-        prog_fn <- paste0(out.fn, ".progress")
+        prog_fn <- paste0(out.fn, ".progress.txt")
         progfile <- file(prog_fn, "wt")
         cat(">>> ", out.fn, " <<<\n", file=progfile, sep="")
         if (verbose)
-            cat("    [Progress Info: ", basename(prog_fn), "]\n", sep="")
+            .cat("    [Progress Info: ", basename(prog_fn), "]")
         infile <- NULL
         on.exit({
             close(progfile)
-            unlink(paste0(out.fn, ".progress"), force=TRUE)
+            unlink(prog_fn, force=TRUE)
             if (!is.null(infile)) close(infile)
         }, add=TRUE)
 
@@ -1253,7 +1361,7 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
         for (fn in ptmpfn)
         {
             if (verbose)
-                cat("    ", basename(fn), " ...", sep="")
+                cat("    ", basename(fn), sep="")
             # open the gds file
             tmpgds <- seqOpen(fn, allow.duplicate=TRUE)
             # merge variables
@@ -1269,7 +1377,7 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
             # close the file
             seqClose(tmpgds)
             if (verbose)
-                cat(" [done]\n")
+                .cat(" [done, ", .tm(), "]")
         }
 
         filtervar <- as.factor(filtervar)
@@ -1324,20 +1432,14 @@ seqVCF2GDS <- function(vcf.fn, out.fn, header=NULL,
     # optimize access efficiency
 
     if (verbose)
-    {
-        cat("Done.\n")
-        cat(date(), "\n", sep="")
-    }
+        if (optimize) .cat("Done.  # ", .tm()) else cat("Done.\n")
     if (optimize)
     {
         if (verbose)
-        {
             cat("Optimize the access efficiency ...\n")
-            flush.console()
-        }
         cleanup.gds(out.fn, verbose=verbose)
-        if (verbose) cat(date(), "\n", sep="")
     }
+    if (verbose && show_timeheader) .cat("##> ", .tm())
 
     # output
     invisible(normalizePath(out.fn))
