@@ -2,7 +2,7 @@
 //
 // GetData.cpp: Get data from a SeqArray GDS file
 //
-// Copyright (C) 2015-2024    Xiuwen Zheng
+// Copyright (C) 2015-2025    Xiuwen Zheng
 //
 // This file is part of SeqArray.
 //
@@ -58,6 +58,7 @@ static const string VAR_GENO_INDEX("@genotype");
 static const string VAR_PHASE("phase");
 
 // variable list: internally generated
+static const string VAR_CHROM_RLE("$chromosome");
 static const string VAR_DOSAGE("$dosage");
 static const string VAR_DOSAGE_ALT("$dosage_alt");
 static const string VAR_DOSAGE_ALT2("$dosage_alt2");
@@ -152,6 +153,82 @@ static SEXP get_chrom(CFileInfo &File, TVarMap &Var, void *param)
 	return rv_ans;
 }
 
+
+// Using the S4Vectors package for RLE-coded chromosome
+
+extern "C"
+{
+	SEXP LANG_NEW_RLE = NULL;
+}
+
+/// create a S4Vectors::Rle object
+inline static SEXP new_s4vectors_rle(const vector<string> &val,
+	const vector<int> &len)
+{
+	// LANG_NEW_RLE = quote(new("Rle", values=values, lengths=lengths))
+	SEXP epr = PROTECT(LANG_NEW_RLE);
+	const int n = val.size();
+	// parameter: values
+	SEXP var_val = PROTECT(NEW_CHARACTER(n));
+	if (n > 0)
+	{
+		for (int i=0; i < n; i++)
+		{
+			const string &s = val[i];
+			SET_STRING_ELT(var_val, i, Rf_mkCharLen(s.c_str(), s.size()));
+		}
+	}
+	// parameter: lengths
+	SEXP var_len = PROTECT(NEW_INTEGER(n));
+	if (n > 0)
+		memcpy(INTEGER(var_len), &len[0], sizeof(int)*n);
+	// set parameters
+	SETCADDR(epr, var_val);   // parameter: values
+	SETCADDDR(epr, var_len);  // parameter: lengths
+	// call
+	SEXP rv = PROTECT(Rf_eval(epr, R_GlobalEnv));
+	// free
+	SETCADDR(epr, R_NilValue);   // parameter: values
+	SETCADDDR(epr, R_NilValue);  // parameter: lengths
+	UNPROTECT(4);
+	// output
+	return rv;
+}
+
+/// get RLE-coded chromosome from '$chromosome'
+static SEXP get_chrom_rle(CFileInfo &File, TVarMap &Var, void *param)
+{
+	int n = File.VariantSelNum();
+	vector<int> rle_len;
+	vector<string> rle_val;
+	if (n > 0)
+	{
+		CChromIndex &Chrom = File.Chromosome();
+		TSelection &Sel = File.Selection();
+		C_BOOL *s = Sel.pVariant + Sel.varStart;
+		size_t i = Sel.varStart;
+		string lastS;
+		for (; n > 0; i++)
+		{
+			if (*s++)
+			{
+				const string &ss = Chrom[i];
+				if ((ss != lastS) || rle_len.empty())
+				{
+					rle_len.push_back(1);
+					rle_val.push_back(ss);
+					lastS = ss;
+				} else {
+					rle_len.back()++;
+				}
+				n--;
+			}
+		}
+	}
+	// return S4Vectors::Rle
+	return new_s4vectors_rle(rle_val, rle_len);
+}
+
 /// get variant-specific data from node without indexing (e.g., variant.id)
 static SEXP get_data_1d(CFileInfo &File, TVarMap &Var, void *param)
 {
@@ -752,7 +829,10 @@ static SEXP get_list(SEXP len, SEXP val, size_t elmsize, bool is_factor)
 			}
 			vv = ZeroLen;
 		} else {
-			vv = Rf_allocVector(TYPEOF(val), nn);
+			if (elmsize <= 1)
+				vv = Rf_allocVector(TYPEOF(val), nn);
+			else
+				vv = Rf_allocMatrix(TYPEOF(val), elmsize, psel[i]);
 			if (is_factor)
 			{
 				SET_CLASS(vv, GET_CLASS(val));
@@ -781,7 +861,7 @@ static SEXP get_list(SEXP len, SEXP val, size_t elmsize, bool is_factor)
 				memcpy(RAW(vv), &RAW(val)[pt], nn);
 				break;
 			default:
-				throw ErrSeqArray("Not support data type for .tolist=TRUE.");
+				throw ErrSeqArray("Not support data type when '.tolist=TRUE'.");
 			}
 			pt += nn;
 		}
@@ -789,6 +869,66 @@ static SEXP get_list(SEXP len, SEXP val, size_t elmsize, bool is_factor)
 	return rv_ans;
 }
 
+
+extern "C"
+{
+	// .List_IRanges_value
+	SEXP OBJ_CompressedList = NULL;
+	// as(, "List")
+	SEXP LANG_AS_LIST = NULL;
+}
+
+/// create a S4Vectors::List object
+inline static SEXP as_s4vectors_list(SEXP x)
+{
+	// LANG_AS_LIST = quote(as(object, "List"))
+	SEXP epr = PROTECT(LANG_AS_LIST);
+	// set parameters
+	SETCADR(epr, x);   // parameter: object
+	// call
+	SEXP rv = PROTECT(Rf_eval(epr, R_GlobalEnv));
+	// free
+	SETCADR(epr, R_NilValue);   // parameter: object
+	// output
+	UNPROTECT(2);
+	return rv;
+}
+
+/// convert to a compressed list in IRanges
+static SEXP get_list2(SEXP len, SEXP val, bool is_factor)
+{
+	// get data type
+	int type_idx = -1;
+	switch (TYPEOF(val))
+	{
+		case LGLSXP:  type_idx = 0; break;
+		case INTSXP:  type_idx = 1; break;
+		case REALSXP: type_idx = 2; break;
+		case STRSXP:  type_idx = 3; break;
+		case RAWSXP:  type_idx = 4; break;
+		default:
+			throw ErrSeqArray("Not support data type when '.tolist=NA'.");
+	}
+	if (is_factor) type_idx = 5;
+	// new object
+	SEXP rv_ans = Rf_duplicate(VECTOR_ELT(OBJ_CompressedList, type_idx));
+	PROTECT(rv_ans);
+	// set @unlistData (OBJ_CompressedList[6]="unlistData")
+	R_do_slot_assign(rv_ans, VECTOR_ELT(OBJ_CompressedList, 6), val);
+	// get @partitioning (OBJ_CompressedList[7]="partitioning")
+	SEXP pt = R_do_slot(rv_ans, VECTOR_ELT(OBJ_CompressedList, 7));
+	// set @partitioning@end (OBJ_CompressedList[8]="end")
+	const int n = Rf_length(len);
+	SEXP end = PROTECT(NEW_INTEGER(n));
+	const int *s = INTEGER(len);
+	int *p = INTEGER(end), ed = 0;
+	for (int i=0; i < n; i++) p[i] = (ed += s[i]);
+	R_do_slot_assign(pt, VECTOR_ELT(OBJ_CompressedList, 8), end);
+	UNPROTECT(1);
+	// output
+	return rv_ans;
+}
+
 /// get data from annotation/info/VARIABLE, TODO
 static SEXP get_info(CFileInfo &File, TVarMap &Var, void *param)
 {
@@ -895,11 +1035,17 @@ static SEXP get_info(CFileInfo &File, TVarMap &Var, void *param)
 			default:
 				throw ErrSeqArray("Not support data type for .padNA=TRUE.");
 			}
-		} else if (P->tolist)
+		} else if (P->tolist)  // TRUE or NA
 		{
 			// convert to a list
 			size_t d2 = (Var.NDim < 2) ? 1 : dimcnt[1];
-			rv_ans = get_list(I32, val, d2, is_factor);
+			if ((P->tolist == TRUE) || (d2 != 1))
+			{
+				rv_ans = get_list(I32, val, d2, is_factor);
+				if (P->tolist == NA_INTEGER)
+					rv_ans = as_s4vectors_list(rv_ans);
+			} else
+				rv_ans = get_list2(I32, val, is_factor);
 		} else {
 			// create `list(length, data)`
 			rv_ans = PROTECT(NEW_LIST(2));
@@ -1141,7 +1287,11 @@ COREARRAY_DLL_LOCAL TVarMap &VarGetStruct(CFileInfo &File, const string &name)
 		} else
 		// ===========================================================
 		// internally generated variables
-		if (name == VAR_DOSAGE)
+		if (name == VAR_CHROM_RLE)
+		{
+			vm.Init(File, VAR_CHROM, get_chrom_rle);
+			CHECK_VARIANT_ONE_DIMENSION
+		} else if (name == VAR_DOSAGE)
 		{
 			vm.Func = get_dosage;
 		} else if (name == VAR_DOSAGE_ALT)
@@ -1291,9 +1441,9 @@ COREARRAY_DLL_EXPORT SEXP SEQ_GetData(SEXP gdsfile, SEXP var_name, SEXP UseRaw,
 	if (padNA == NA_LOGICAL)
 		Rf_error("'.padNA' must be TRUE or FALSE.");
 	// .tolist
+	if (!Rf_isLogical(ToList) || Rf_length(ToList)!=1)
+		Rf_error("'.tolist' must be TRUE, FALSE or NA.");
 	const int tolist = Rf_asLogical(ToList);
-	if (tolist == NA_LOGICAL)
-		Rf_error("'.tolist' must be TRUE or FALSE.");
 	// .envir
 	if (!Rf_isNull(Env))
 	{
@@ -1350,12 +1500,13 @@ COREARRAY_DLL_EXPORT SEXP SEQ_BApply_Variant(SEXP gdsfile, SEXP var_name,
 		Rf_error("'.padNA' must be TRUE or FALSE.");
 	// .tolist
 	int tolist = Rf_asLogical(RGetListElement(param, "tolist"));
-	if (tolist == NA_LOGICAL)
-		Rf_error("'.tolist' must be TRUE or FALSE.");
+	// if (tolist == NA_LOGICAL)
+	//	Rf_error("'.tolist' must be TRUE or FALSE.");
 	// .progress
 	int prog_flag = Rf_asLogical(RGetListElement(param, "progress"));
 	if (prog_flag == NA_LOGICAL)
 		Rf_error("'.progress' must be TRUE or FALSE.");
+	SEXP prog_file = RGetListElement(param, "progressfile");
 
 	COREARRAY_TRY
 
@@ -1458,7 +1609,7 @@ COREARRAY_DLL_EXPORT SEXP SEQ_BApply_Variant(SEXP gdsfile, SEXP var_name,
 		pEnd = pBase + File.VariantNum();
 
 		// progress object
-		CProgressStdOut progress(NumBlock, 1, prog_flag);
+		CProgress progress(NumBlock, prog_file, prog_flag);
 
 		// for-loop
 		for (int idx=0; idx < NumBlock; idx++)
